@@ -3,39 +3,40 @@
 # changes made to this file will be lost the next time ruby is built.
 
 module RbConfig
-  RUBY_VERSION == "2.0.0" or
-    raise "ruby lib version (2.0.0) doesn't match executable version (#{RUBY_VERSION})"
+  RUBY_VERSION == "2.1.0" or
+    raise "ruby lib version (2.1.0) doesn't match executable version (#{RUBY_VERSION})"
 
-  TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/2.0/sparc64-openbsd")
+  TOPDIR = File.dirname(__FILE__).chomp!("/lib/ruby/2.1/sparc64-openbsd")
   DESTDIR = '' unless defined? DESTDIR
   CONFIG = {}
   CONFIG["DESTDIR"] = DESTDIR
   CONFIG["MAJOR"] = "2"
-  CONFIG["MINOR"] = "0"
+  CONFIG["MINOR"] = "1"
   CONFIG["TEENY"] = "0"
-  CONFIG["PATCHLEVEL"] = "648"
-  CONFIG["INSTALL"] = '/usr/obj/ports/ruby-2.0.0-p648/bin/install -c '
+  CONFIG["PATCHLEVEL"] = "0"
+  CONFIG["INSTALL"] = '/usr/bin/install -c -o root -g bin'
   CONFIG["EXEEXT"] = ""
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
-  CONFIG["ruby_install_name"] = "ruby20"
-  CONFIG["RUBY_INSTALL_NAME"] = "ruby20"
-  CONFIG["RUBY_SO_NAME"] = "ruby20"
+  CONFIG["ruby_install_name"] = "ruby21"
+  CONFIG["RUBY_INSTALL_NAME"] = "ruby21"
+  CONFIG["RUBY_SO_NAME"] = "ruby21"
   CONFIG["exec"] = "exec"
-  CONFIG["ruby_pc"] = "ruby-2.0.pc"
+  CONFIG["ruby_pc"] = "ruby-2.1.pc"
   CONFIG["PACKAGE"] = "ruby"
   CONFIG["BUILTIN_TRANSSRCS"] = " newline.c"
   CONFIG["USE_RUBYGEMS"] = "YES"
-  CONFIG["MANTYPE"] = "doc"
-  CONFIG["NROFF"] = ""
+  CONFIG["MANTYPE"] = "man"
+  CONFIG["NROFF"] = "/bin/false"
   CONFIG["vendorarchhdrdir"] = "$(vendorhdrdir)/$(sitearch)"
   CONFIG["sitearchhdrdir"] = "$(sitehdrdir)/$(sitearch)"
   CONFIG["rubyarchhdrdir"] = "$(rubyhdrdir)/$(arch)"
   CONFIG["vendorhdrdir"] = "$(rubyhdrdir)/vendor_ruby"
   CONFIG["sitehdrdir"] = "$(rubyhdrdir)/site_ruby"
   CONFIG["rubyhdrdir"] = "$(includedir)/$(RUBY_VERSION_NAME)"
+  CONFIG["RUBY_SEARCH_PATH"] = ""
   CONFIG["UNIVERSAL_INTS"] = ""
   CONFIG["UNIVERSAL_ARCHNAMES"] = ""
-  CONFIG["configure_args"] = " '--program-suffix=20' '--with-soname=ruby20' '--with-ruby-version=minor' '--with-mantype=doc' '--enable-pthread' '--enable-ipv6' '--disable-option-checking' '--enable-shared' '--with-tcl-include=/usr/local/include/tcl8.5' '--with-tk-include=/usr/local/include/tk8.5' '--with-tcllib=tcl85' '--with-tklib=tk85' '--with-X11-dir=/usr/X11R6' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' '--disable-gtk-doc' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXX=c++' 'CXXFLAGS=-O2 -pipe'"
+  CONFIG["configure_args"] = " '--program-suffix=21' '--with-soname=ruby21' '--with-ruby-version=minor' '--enable-pthread' '--enable-ipv6' '--disable-option-checking' '--with-tcl-include=/usr/local/include/tcl8.5' '--with-tk-include=/usr/local/include/tk8.5' '--with-tcllib=tcl85' '--with-tklib=tk85' '--with-X11-dir=/usr/X11R6' '--enable-shared' '--prefix=/usr/local' '--sysconfdir=/etc' '--mandir=/usr/local/man' '--infodir=/usr/local/info' '--localstatedir=/var' '--disable-silent-rules' 'CC=cc' 'CFLAGS=-O2 -pipe' 'LDFLAGS=-L/usr/local/lib' 'CPPFLAGS=-DOPENSSL_NO_STATIC_ENGINE -I/usr/local/include' 'CXX=c++' 'CXXFLAGS=-O2 -pipe'"
   CONFIG["vendorarchdir"] = "$(vendorlibdir)/$(sitearch)"
   CONFIG["vendorlibdir"] = "$(vendordir)/$(ruby_version)"
   CONFIG["vendordir"] = "$(rubylibprefix)/vendor_ruby"
@@ -44,18 +45,21 @@ module RbConfig
   CONFIG["sitedir"] = "$(rubylibprefix)/site_ruby"
   CONFIG["rubyarchdir"] = "$(rubylibdir)/$(arch)"
   CONFIG["rubylibdir"] = "$(rubylibprefix)/$(ruby_version)"
-  CONFIG["ruby_version"] = "2.0"
+  CONFIG["ruby_version"] = "2.1"
   CONFIG["sitearch"] = "$(arch)"
   CONFIG["arch"] = "sparc64-openbsd"
   CONFIG["sitearchincludedir"] = "$(includedir)/$(sitearch)"
   CONFIG["archincludedir"] = "$(includedir)/$(arch)"
   CONFIG["sitearchlibdir"] = "$(libdir)/$(sitearch)"
   CONFIG["archlibdir"] = "$(libdir)/$(arch)"
+  CONFIG["libdirname"] = "libdir"
+  CONFIG["RUBY_EXEC_PREFIX"] = "/usr/local"
+  CONFIG["RUBY_LIB_VERSION"] = ""
+  CONFIG["RUBY_LIB_VERSION_STYLE"] = "2\t/* minor */"
   CONFIG["RI_BASE_NAME"] = "ri"
   CONFIG["ridir"] = "$(datarootdir)/$(RI_BASE_NAME)"
   CONFIG["rubysitearchprefix"] = "$(rubylibprefix)/$(sitearch)"
   CONFIG["rubyarchprefix"] = "$(rubylibprefix)/$(arch)"
-  CONFIG["rubylibprefix"] = "$(libdir)/$(RUBY_BASE_NAME)"
   CONFIG["MAKEFILES"] = "Makefile"
   CONFIG["PLATFORM_DIR"] = ""
   CONFIG["THREAD_MODEL"] = "pthread"
@@ -73,12 +77,10 @@ module RbConfig
   CONFIG["LIBRUBYARG"] = "$(LIBRUBYARG_SHARED)"
   CONFIG["LIBRUBY"] = "$(LIBRUBY_SO)"
   CONFIG["LIBRUBY_ALIASES"] = "lib$(RUBY_SO_NAME).so"
-  CONFIG["LIBRUBY_SO"] = "lib$(RUBY_SO_NAME).so.1.1"
+  CONFIG["LIBRUBY_SO"] = "lib$(RUBY_SO_NAME).so.0.0"
   CONFIG["LIBRUBY_A"] = "lib$(RUBY_SO_NAME)-static.a"
   CONFIG["RUBYW_INSTALL_NAME"] = ""
   CONFIG["rubyw_install_name"] = ""
-  CONFIG["LIBRUBY_DLDFLAGS"] = ""
-  CONFIG["LIBRUBY_LDSHARED"] = "$(CC) -shared $(CCDLFLAGS)"
   CONFIG["EXTDLDFLAGS"] = ""
   CONFIG["EXTLDFLAGS"] = ""
   CONFIG["strict_warnflags"] = "-ansi -std=iso9899:199409"
@@ -105,6 +107,7 @@ module RbConfig
   CONFIG["BTESTRUBY"] = "$(MINIRUBY)"
   CONFIG["CROSS_COMPILING"] = "no"
   CONFIG["TEST_RUNNABLE"] = "yes"
+  CONFIG["rubylibprefix"] = "$(libdir)/$(RUBY_BASE_NAME)"
   CONFIG["setup"] = "Setup"
   CONFIG["EXTSTATIC"] = ""
   CONFIG["STRIP"] = "strip"
@@ -121,7 +124,7 @@ module RbConfig
   CONFIG["CCDLFLAGS"] = "-fPIC"
   CONFIG["STATIC"] = ""
   CONFIG["ARCH_FLAG"] = ""
-  CONFIG["DLDFLAGS"] = ""
+  CONFIG["DLDFLAGS"] = "-L/usr/local/lib"
   CONFIG["ALLOCA"] = ""
   CONFIG["codesign"] = ""
   CONFIG["POSTLINK"] = ":"
@@ -139,9 +142,9 @@ module RbConfig
   CONFIG["DTRACE"] = ""
   CONFIG["MAKEDIRS"] = "mkdir -p"
   CONFIG["MKDIR_P"] = "mkdir -p"
-  CONFIG["INSTALL_DATA"] = "/usr/obj/ports/ruby-2.0.0-p648/bin/install -c  -m 644"
-  CONFIG["INSTALL_SCRIPT"] = "/usr/obj/ports/ruby-2.0.0-p648/bin/install -c  -m 755"
-  CONFIG["INSTALL_PROGRAM"] = "/usr/obj/ports/ruby-2.0.0-p648/bin/install -c -s  -m 755"
+  CONFIG["INSTALL_DATA"] = "install -c -o root -g bin -m 444"
+  CONFIG["INSTALL_SCRIPT"] = "install -c -o root -g bin -m 555"
+  CONFIG["INSTALL_PROGRAM"] = "install -c -s -o root -g bin -m 555"
   CONFIG["SET_MAKE"] = ""
   CONFIG["LN_S"] = "ln -s"
   CONFIG["NM"] = "nm"
@@ -154,6 +157,7 @@ module RbConfig
   CONFIG["AR"] = "ar"
   CONFIG["RANLIB"] = "ranlib"
   CONFIG["try_header"] = ""
+  CONFIG["CC_VERSION"] = "$(CC) -v"
   CONFIG["COUTFLAG"] = "-o "
   CONFIG["OUTFLAG"] = "-o "
   CONFIG["CPPOUTFILE"] = "-o conftest.i"
@@ -173,27 +177,27 @@ module RbConfig
   CONFIG["NACL_SDK_VARIANT"] = ""
   CONFIG["NACL_SDK_ROOT"] = ""
   CONFIG["NACL_TOOLCHAIN"] = ""
-  CONFIG["target_os"] = "openbsd6.0"
+  CONFIG["target_os"] = "openbsd5.5"
   CONFIG["target_vendor"] = "unknown"
   CONFIG["target_cpu"] = "sparc64"
-  CONFIG["target"] = "sparc64-unknown-openbsd6.0"
-  CONFIG["host_os"] = "openbsd6.0"
+  CONFIG["target"] = "sparc64-unknown-openbsd5.5"
+  CONFIG["host_os"] = "openbsd5.5"
   CONFIG["host_vendor"] = "unknown"
   CONFIG["host_cpu"] = "sparc64"
-  CONFIG["host"] = "sparc64-unknown-openbsd6.0"
+  CONFIG["host"] = "sparc64-unknown-openbsd5.5"
   CONFIG["RUBY_VERSION_NAME"] = "$(RUBY_BASE_NAME)-$(ruby_version)"
   CONFIG["RUBYW_BASE_NAME"] = "rubyw"
   CONFIG["RUBY_BASE_NAME"] = "ruby"
-  CONFIG["build_os"] = "openbsd6.0"
+  CONFIG["build_os"] = "openbsd5.5"
   CONFIG["build_vendor"] = "unknown"
   CONFIG["build_cpu"] = "sparc64"
-  CONFIG["build"] = "sparc64-unknown-openbsd6.0"
-  CONFIG["RUBY_RELEASE_DATE"] = "2015-12-16"
-  CONFIG["RUBY_PROGRAM_VERSION"] = "2.0.0"
+  CONFIG["build"] = "sparc64-unknown-openbsd5.5"
+  CONFIG["RUBY_RELEASE_DATE"] = "2013-12-25"
+  CONFIG["RUBY_PROGRAM_VERSION"] = "2.1.0"
   CONFIG["target_alias"] = ""
   CONFIG["host_alias"] = ""
   CONFIG["build_alias"] = ""
-  CONFIG["LIBS"] = "-pthread -lm "
+  CONFIG["LIBS"] = "-pthread -lgmp -lm "
   CONFIG["ECHO_T"] = ""
   CONFIG["ECHO_N"] = "-n"
   CONFIG["ECHO_C"] = ""
@@ -217,7 +221,7 @@ module RbConfig
   CONFIG["libexecdir"] = "$(exec_prefix)/libexec"
   CONFIG["sbindir"] = "$(exec_prefix)/sbin"
   CONFIG["bindir"] = "$(exec_prefix)/bin"
-  CONFIG["program_transform_name"] = "s&$$&20&;s&^&&"
+  CONFIG["program_transform_name"] = "s&$$&21&;s&^&&"
   CONFIG["exec_prefix"] = "$(prefix)"
   CONFIG["PACKAGE_URL"] = ""
   CONFIG["PACKAGE_BUGREPORT"] = ""
@@ -263,23 +267,3 @@ module RbConfig
 end
 autoload :Config, "rbconfig/obsolete.rb" # compatibility for ruby-1.8.4 and older.
 CROSS_COMPILING = nil unless defined? CROSS_COMPILING
-install = '/usr/bin/install -c -o root -g bin'  
-c = RbConfig::CONFIG 
-m = RbConfig::MAKEFILE_CONFIG
-if v = ENV['BSD_INSTALL_SCRIPT']
-  m['INSTALL'] = c['INSTALL'] = v.sub('-m 755', '')
-  m['INSTALL_SCRIPT'] = c['INSTALL_SCRIPT'] = v
-else
-  m['INSTALL'] = c['INSTALL'] = "#{install}"
-  m['INSTALL_SCRIPT'] = c['INSTALL_SCRIPT'] = "#{install} -m 755"
-end
-if v = ENV['BSD_INSTALL_PROGRAM']
-  m['INSTALL_PROGRAM'] = c['INSTALL_PROGRAM'] = v
-else
-  m['INSTALL_PROGRAM'] = c['INSTALL_PROGRAM'] = "#{install} -s -m 755"
-end
-if v = ENV['BSD_INSTALL_DATA']
-  m['INSTALL_DATA'] = c['INSTALL_DATA'] = v
-else
-  m['INSTALL_DATA'] = c['INSTALL_DATA'] = "#{install} -m 644"
-end
